unit ThingHolderUnit;

interface
uses Classes, SysUtils, ThingUnit;

type
ThingHolder = class(Thing)
  protected
  //  _things: TList;
  public
    _things: TList; // TODO: Rewrite with accessor method or property
    constructor Create(aName, aDescription: shortstring);
    destructor Destroy; override;
    function GetThings: TList;
    procedure SetThings(someThings: TList);
    procedure AddThing(aThing: Thing);
    function NameInThingsAt(aName: string): integer;
    function ObInThings(aName: string): Thing;
    procedure TakeObFrom(var aThing: Thing; var tl: TList);
    procedure WriteToStream(fs: TFileStream); override;
    procedure ReadFromStream(fs: TFileStream); override;
  end;

implementation
uses RoomUnit, ActorUnit;

constructor ThingHolder.Create(aName, aDescription: shortstring);
begin
  inherited Create(aName, aDescription);
  _things := TList.Create;
end;

destructor ThingHolder.Destroy;
var
  i: integer;
begin
  for i := 0 to _things.Count - 1 do
  begin
    if _things.Items[i] <> nil then
      Thing(_things.Items[i]).Free;
  end;
  _things.Clear;
  _things.Free;
  inherited Destroy;
end;

function ThingHolder.GetThings: TList;
begin
  result := _things;
end;

procedure ThingHolder.SetThings(someThings: TList);
begin
  _things := someThings;
end;

procedure ThingHolder.AddThing(aThing: Thing);
begin
  _things.Add(aThing);
end;

procedure ThingHolder.TakeObFrom(var aThing: Thing; var tl: TList);
begin
  _things.Add(aThing);
  tl.Remove(aThing);
end;

function ThingHolder.NameInThingsAt(aName: string): integer;
// return index in things of object with name aName or -1 if not found
var
  i, index: integer;
begin
  index := -1;
  i := 0;
  aName := lowercase(aName);
  while ((i < _things.Count) and (index = -1)) do
    if lowercase(Thing(_things.Items[i]).Name) = aName then
      index := i
    else
      i := i + 1;
  result := index;
end;

function ThingHolder.ObInThings(aName: string): Thing;
// return object in things with name n or NIL if not found
var
  i: integer;
  ob: Thing;
begin
  ob := nil;
  i := 0;
  aName := lowercase(aName);
  while ((i < _things.Count) and (ob = nil)) do
  begin
    if lowercase(Thing(_things.Items[i]).Name) = aName then
      ob := Thing(_things.Items[i])
    else
      i := i + 1;
  end;
  result := ob;
end;

procedure ThingHolder.WriteToStream(fs: TFileStream);
var
  i: integer;
begin
  inherited WriteToStream(fs);
  fs.WriteBuffer(_things.Count, sizeof(_things.Count)); // save num of things
  for i := 0 to _things.Count - 1 do
    Thing(_things[i]).WriteToStream(fs);
end;

procedure ThingHolder.ReadFromStream(fs: TFileStream);
var
  i: integer;
  numthings: integer;
  currentClassName: shortstring;
begin
  inherited ReadFromStream(fs);
  fs.ReadBuffer(numthings, sizeof(numthings));
  _things := TList.Create; // remember to create a list here!
  for i := 0 to numthings - 1 do
  begin
    fs.ReadBuffer(currentClassName, sizeof(currentClassName));
    if currentClassName = 'Thing' then
      _things.Add(Thing.CreateFromStream(fs))
    else if currentClassName = 'ThingHolder' then
      _things.Add(ThingHolder.CreateFromStream(fs))
    else if currentClassName = 'Room' then
      _things.Add(Room.CreateFromStream(fs))
    else if currentClassName = 'Actor' then
      _things.Add(Actor.CreateFromStream(fs));
  end;
end;



end.
